#!/usr/bin/python3
import argparse
import json
import os.path
import urllib.parse
import instaloader
import requests

def syncInsta(args) -> None:
    js = {}
    if os.path.exists(args.data):
        print("Try to load ", args.data)
        try:
            with open(args.data, 'rb') as f:
                js = json.load(f)
        except:
            pass

    if "posts" not in js:
        js["posts"] = []

    l = instaloader.Instaloader()
    profile = instaloader.Profile.from_username(l.context, args.profile)
    for post in profile.get_posts():
        shortcode = post.shortcode
        print("Verify if {} exists".format(shortcode))
        if not any(obj['shortcode'] == shortcode for obj in js["posts"]):
            # not found yet.
            entry = {
                "shortcode": shortcode,
                "title": post.title,
                "caption": post.caption,
                "url": post.url,
                "dateCreate": post.date_local.isoformat(),
                "contentType": "",
                "fileName": ""
            }
            fileName = os.path.basename(urllib.parse.urlparse(post.url).path)
            print("Download {} via {} and save as {}".format(shortcode, post.url, fileName))
            req = requests.get(post.url)
            if req.ok:
                entry["contentType"] = req.headers.get('Content-Type')
                with open(os.path.join(args.output, fileName), 'wb') as f:
                    f.write(req.content)
                entry["fileName"] = fileName

            js["posts"].append(entry)

    js["posts"] = sorted(js["posts"], key=lambda x: x["dateCreate"])
    with open(args.data, 'wb') as f:
        f.write(json.dumps(js).encode('utf-8'))

def cli():
    parser = argparse.ArgumentParser()
    parser.add_argument('--profile', dest='profile', default="cvjmrheinlahn", type=str, help='Instagram Profile')
    parser.add_argument('--data', '-d', dest='data', required=True, help='Data file (json)')
    parser.add_argument('--output', '-o', dest='output', required=True, help='Image path')
    return parser.parse_args()

if __name__ == '__main__':
    args = cli()
    syncInsta(args)
