class DynamicContentLoader {
  constructor (listElement) {
    this.listElement = listElement
    this.container = listElement.parentNode
    this.footer = this.container.querySelector('.list-footer')
    this.hasNext = false
    this.nextPageUrlTemplate = this._getNextPageTemplate()
    this.isMasonry = listElement.classList.contains('masonry')
    this.msnry = null
    this.data = null
    this.steps = this._getSteps()
    this._addEventListener()
    this._addMasonry()
    this._loadData()
  }

  _addMasonry () {
    if (!this.isMasonry) {
      return
    }
    var dcl = this
    console.log('Install imagesLoaded')
    imagesLoaded(this.listElement, function () {
      console.log('Images are all loaded!')
      // init Isotope after all images have loaded
      dcl.msnry = new Masonry(dcl.listElement, {
        itemSelector: '.masonry-item',
        columnWidth: 375,
        gutter: 60,
        percentPosition: false
      })
    })
  }

  _getSteps () {
    const steps = Number.parseInt(
      this.listElement.getAttribute('data-list-step')
    )
    if (!isNaN(steps) && steps > 0) {
      return steps
    } else {
      return this._getOffset()
    }
  }

  _getNextPageTemplate () {
    const dnmb = this.footer.getElementsByClassName('dynamic-more')
    if (dnmb && dnmb.length > 0) {
      return dnmb[0].getAttribute('data-page-tpl')
    }
    return null
  }

  _getOffset () {
    return this.listElement.children.length
  }

  _loadData () {
    const baseURL = this.listElement.getAttribute('data-section-url')
    if (baseURL) {
      const url = baseURL + (baseURL.endsWith('/') ? '' : '/') + 'index.json'
      var dcl = this
      const xhttp = new XMLHttpRequest()
      xhttp.onload = function () {
        dcl.data = JSON.parse(this.response).data
        console.log(dcl.data)
      }
      xhttp.open('GET', url)
      xhttp.send()
    }
  }

  _addEventListener () {
    // Only, if there is something.
    if (this.listElement.getAttribute('data-has-next') !== 'true') {
      return
    }
    this.hasNext = true

    var dcl = this
    Array.from(this.footer.getElementsByClassName('dynamic-more')).forEach(
      moreButton => {
        moreButton.addEventListener('click', event => {
          if (dcl.data) {
            dcl._handleJsonList(event)
          } else {
            dcl._handleHtmlList(event)
          }
        })
      }
    )
  }

  _handleHtmlList (event) {
    const dcl = this
    const nextPage = Number.parseInt(
      event.target.getAttribute('data-next-page')
    )
    if (!isNaN(nextPage)) {
      const url = this.nextPageUrlTemplate.replace(':page:', nextPage)
      const xhttp = new XMLHttpRequest()
      xhttp.onload = function () {
        var div = document.createElement('div')
        div.innerHTML = this.response
        var mplist = div.getElementsByClassName('dynamic-list')
        if (mplist.length > 0) {
          mplist = mplist[0]
          Array.from(mplist.children).forEach(entry => {
            dcl._addEntry(entry)
          })
        }
        // Check for any pagination
        var pn = mplist.parentNode.querySelector('.pagination-next')
        if (pn) {
          var nextPageNumber = Number.parseInt(
            pn.getAttribute('data-next-page')
          )
          if (!isNaN(nextPageNumber)) {
            event.target.setAttribute('data-next-page', nextPageNumber)
          }
        } else {
          // Nothing more. Delete node.
          event.target.parentNode.removeChild(event.target)
        }
      }
      xhttp.open('GET', url, true)
      xhttp.send()
    }
  }

  _handleJsonList (event) {
    console.log('Handle json', !this.data)
    if (!this.data) {
      return
    }
    const curOffset = this._getOffset()
    const endOffset = curOffset + this.steps

    this.data.items.slice(curOffset, endOffset).forEach(data => {
      // Prepare element.
      const rawEntry = this._decode(data.represent)
      const entry = document.createElement('div')
      entry.innerHTML = rawEntry
      this._addEntry(entry.firstChild)
    })

    // end reached? Remove "+" sign.
    if (this._getOffset() >= this.data.count) {
      event.target.parentNode.removeChild(event.target)
    }
  }

  _decode (html) {
    var txt = document.createElement('textarea')
    txt.innerHTML = html
    return txt.value
  }

  _addEntry (entry) {
    this.listElement.appendChild(entry)
    if (this.isMasonry && this.msnry) {
      this.msnry.appended(entry)
    }
  }
}

function closeModal() {
  var overlay = $('#modal-overlay');
  overlay.css('display', 'none');
}
function dynamicify () {
  console.log('Lets start!')
  Array.from(document.getElementsByClassName('dynamic-list')).forEach(
    element => {
      new DynamicContentLoader(element)
    }
  )

  // Popups / modals
  $('div.event-modal td.blgt').on('click', function() {
    var overlay = $('#modal-overlay');
    var content = $(this).children('.modal-overlay');
    // reset events if already set
    overlay.off('click', '.close', closeModal);
    overlay.html(content.html());
    overlay.css('display', 'block');
    overlay.on('click', '.close', closeModal);
  });

  $(window).on('click', function(event) {
    var overlay = $('#modal-overlay');
    if (overlay.is(event.target)) {
      closeModal();
    }
  });


  Array.from(document.getElementsByClassName('slider-container')).forEach(
    element => {
        var splideId = element.getAttribute('data-splide-id');
        if (splideId !== null) {
            var type = 'slide';
            var arrows = false;
            if (element.getElementsByClassName('splide__slide').length > 2) {
                type = 'loop';
            }
            if (element.getElementsByClassName('splide__slide').length > 1) {
                type = 'loop';
                arrows = true;
            }
            // Force type?
            if (element.hasAttribute('data-splide-type') && element.getAttribute('data-splide-type').length > 0) {
                type = element.getAttribute('data-splide-type');
            }
            var splide = new Splide(element, {
                type: type,
                //padding: '5rem',
                gap: '20px',
                pagination: false,
                autoWidth: true,
                autoHeight: true,
                wheel: false,
                direction: 'ltr',
                arrows: arrows
            });
            var bar = splide.root.querySelector('.slider-progress-bar')
            splide.on('mounted move', function () {
                var end = splide.Components.Controller.getEnd() + 1
                var rate = Math.min((splide.index + 1) / end, 1)
                bar.style.width = String(100 * rate) + '%'
            });
            
            splide.mount()
        }
    }
  );
}

// listen for load event in the window
window.addEventListener('load', function () {
  // do things after the DOM loads fully
  console.log('Everything is loaded')
  dynamicify()
})
